@@ -1,9 +1,9 @@
 ## -*- coding: utf-8 -*-
-#'''
-#Created on 12/giu/2009
-#
-#@author: sax
-#'''
+'''
+Created on 12/giu/2009
+
+@author: sax
+'''
 
 from django.contrib import admin
 from django.core.urlresolvers import reverse
@@ -117,7 +117,7 @@ class ConcurrentModelAdmin(admin.ModelAdmin):
                 return HttpResponseRedirect(".")
 
     def get_urls(self):
-        from django.conf.urls import patterns, url, include
+        from django.conf.urls import patterns, url
 
         def wrap(view):
             def wrapper(*args, **kwargs):
