@@ -1,34 +1,51 @@
 # -*- coding: utf-8 -*-
-'''
+"""
 Created on 09/giu/2009
 
 @author: sax
-'''
+"""
 import logging
 import time
 import datetime
+from django.contrib.auth.models import User
 from django.forms.models import modelform_factory
 from django.test import TestCase
 from concurrency.core import RecordModifiedError
-from .models import *
+from concurrency.core import apply_concurrency_check
+from concurrency.fields import IntegerVersionField
+from concurrency.forms import VersionFieldSigner
+from concurrency.tests.models import TestModel0, TestModel1, TestModel2, TestModel3, TestModel0_Proxy, \
+    TestAbstractModel0, TestModelGroup, TestModelWithCustomSave, TestIssue3Model, ModelWithCustomSave, \
+    TestModelGroupWithCustomSave, AutoIncConcurrentModel, TestCustomUser
+from concurrency.utils import ConcurrencyTestMixin
 
 
-logger = logging.getLogger('tests.concurrency')
-logger.setLevel(logging.DEBUG)
+logger = logging.getLogger('concurrency.tests')
 
 
-class ConcurrencyError(Exception):
-    pass
+class ConcurrencyTest0(ConcurrencyTestMixin, TestCase):
+    concurrency_model = TestModel0
+    concurrency_kwargs = {'username': 'test'}
 
-
-class ConcurrencyTest0(TestCase):
     def setUp(self):
         super(ConcurrencyTest0, self).setUp()
         self._unique_field_name = 'username'
+        self._get_target()
+        setattr(self.TARGET.__class__, '_get_test_revision_number',
+                ConcurrencyTest0._get_REVISION_NUMBER)
+
+    def _get_target(self):
         self.TARGET = TestModel0(username="New", last_name="1")
 
+    @staticmethod
+    def _get_REVISION_NUMBER(obj):
+        revision_field = obj.RevisionMetaInfo.field
+        value = getattr(obj, revision_field.attname)
+        return value
+
     def _check_save(self, obj):
-        return obj.save()
+        obj.save()
+        assert obj.pk is not None # sanity check
 
     def _get_form_data(self, **kwargs):
         data = {}
@@ -36,48 +53,47 @@ class ConcurrencyTest0(TestCase):
         return data
 
     def test_standard_insert(self):
-        self_version_field_name = self.TARGET._get_revision_field().name
-
         logger.debug("Created Object_1")
-        a = self.TARGET.__class__(username='standard_insert')
-        v = a._get_revision_number()
+        a = self.TARGET.__class__(**self.concurrency_kwargs)
+        v = a._get_test_revision_number()
         logger.debug("Now Object_1.version is %s " % v)
         assert bool(v) is False, "version is not null %s" % v
         a.save()
-        logger.debug("Object_1 saved...now version is %s " % a.version)
-        self.assertTrue(a.pk > 0)
-        v2 = a._get_revision_number()
-        assert v2 != v, "same or lower version after insert (%s,%s)" % (a.version, v)
+        logger.debug("Object_1 saved...now version is %s " % a._get_test_revision_number())
+        self.assertIsNotNone(a.pk)
+        v2 = a._get_test_revision_number()
+        assert v2 != v, "same or lower version after insert (%s,%s)" % (a._get_test_revision_number(), v)
         b = self.TARGET.__class__.objects.get(pk=a.pk)
-        self.assertEqual(a.version, b.version)
+        self.assertEqual(a._get_test_revision_number(), b._get_test_revision_number())
 
     def test_concurrency(self):
         logger.debug("Created Object_1")
         a = self.TARGET
         self._check_save(a)
-        logger.debug("Object_1 saved...now version is %s " % a.version)
-        id = a.pk
+        logger.debug("Object_1 saved...now version is %s " % a._get_test_revision_number())
+        a_pk = a.pk
 
-        a = self.TARGET.__class__.objects.get(pk=id)
-        v = a._get_revision_number()
-        logger.debug("reloaded...now version is %s " % a.version)
+        a = self.TARGET.__class__.objects.get(pk=a_pk)
+        v = a._get_test_revision_number()
+        logger.debug("reloaded...now version is %s " % v)
 
-        b = self.TARGET.__class__.objects.get(pk=id)
-        assert a._get_revision_number() == b._get_revision_number(), "got same row with different version"
+        b = self.TARGET.__class__.objects.get(pk=a_pk)
+        assert a._get_test_revision_number() == b._get_test_revision_number(), "got same row with different version"
         time.sleep(2)
         a.last_name = "pippo"
         self._check_save(a)
-        logger.debug("updated...now version is %s " % a.version)
-        assert a._get_revision_number() != v, "same or lower version after update (%s,%s)" % (a.version, v)
+        logger.debug("updated...now version is %s " % a._get_test_revision_number())
+        assert a._get_test_revision_number() != v, "same version after update (%s,%s)" % (a.version, v)
         self.assertRaises(RecordModifiedError, b.save)
 
     def test_concurrency_no_values(self):
         logger.debug("Created Object_1")
         t = self.TARGET.__class__()
-        assert bool(t._get_revision_number()) is False, "version is not null %s" % t._get_revision_number()
+        assert bool(t._get_test_revision_number()) is False, "version is not null %s" % t._get_test_revision_number()
         t.save()
-        self.assertTrue(t.pk > 0)
-        self.assertGreater(t.version, 0)
+        self.assertIsNotNone(t.pk)
+        self.assertNotEqual(t.version, self.TARGET.RevisionMetaInfo.field.get_default())
+        self.assertTrue(bool(t.version))
 
     def test_force_update(self):
         logger.debug("Created Object_1")
@@ -86,11 +102,11 @@ class ConcurrencyTest0(TestCase):
 
     def test_force_insert(self):
         logger.debug("Created Object_1")
-        t = self.TARGET.__class__(username='empty')
-        assert bool(t._get_revision_number()) is False, "version is not null %s" % t._get_revision_number()
+        t = self.TARGET.__class__(**self.concurrency_kwargs)
+        assert bool(t._get_test_revision_number()) is False, "version is not null %s" % t._get_test_revision_number()
         t.save(force_insert=True)
-        self.assertTrue(t.pk > 0)
-        self.assertTrue(t._get_revision_number())
+        self.assertIsNotNone(t.pk)
+        self.assertTrue(t._get_test_revision_number())
 
     def test_concurrency_manager_list(self):
         logger.debug("Start Test")
@@ -128,63 +144,90 @@ class ConcurrencyTest0(TestCase):
 
     def test_form_save(self):
         formClass = modelform_factory(self.TARGET.__class__)
-        original_version = self.TARGET._get_revision_number()
-        version_field_name = self.TARGET._get_revision_field().name
+        original_version = self.TARGET._get_test_revision_number()
+        version_field_name = self.TARGET.RevisionMetaInfo.field.name
 
-        form = formClass(self._get_form_data(**{version_field_name: original_version}),
+        form = formClass(self._get_form_data(**{version_field_name: VersionFieldSigner().sign(original_version)}),
                          instance=self.TARGET)
 
         self.assertTrue(form.is_valid(), form.errors)
         obj = form.save()
-        self.assertTrue(obj.pk)
-        self.assertGreater(obj._get_revision_number(), original_version)
+        self.assertIsNotNone(obj.pk)
+        # self.assertGreater(obj._get_test_revision_number(), original_version)
+        self.assertNotEqual(obj._get_test_revision_number(), original_version)
 
-        form = formClass(self._get_form_data(**{version_field_name: obj._get_revision_number()}),
+        form = formClass(self._get_form_data(**{version_field_name: VersionFieldSigner().sign(obj._get_test_revision_number())}),
                          instance=obj)
         self.assertTrue(form.is_valid(), form.errors)
-        pre_save_version = obj._get_revision_number()
+        pre_save_version = obj._get_test_revision_number()
         obj_after = form.save()
 
-        self.assertTrue(obj_after.pk)
-        self.assertGreater(obj_after._get_revision_number(), pre_save_version)
+        self.assertIsNotNone(obj_after.pk)
+        self.assertNotEqual(obj_after._get_test_revision_number(), pre_save_version)
+        self.assertTrue(bool(obj_after._get_test_revision_number()), "'%s'" % obj_after._get_test_revision_number())
+
+
+# class DateConcurrencyTest1(ConcurrencyTest0):
+#     concurrency_model = DateConcurrentModel
+#
+#     def _get_target(self):
+#         self.TARGET = DateConcurrentModel(char_field="New", last_name="1")
+
+class AutoIncConcurrencyTest(ConcurrencyTest0):
+    concurrency_model = AutoIncConcurrentModel
+
+    def _get_target(self):
+        self.TARGET = AutoIncConcurrentModel(char_field="New", last_name="1")
+
+    def test_increment(self):
+        logger.debug("Created Object_1")
+        a = self.TARGET
+        self.assertEqual(a.version, 0)
+        self._check_save(a)
+        self.assertEqual(a.version, 1)
+        a.save()
+        self.assertEqual(a.version, 2)
 
 
 class ConcurrencyTest1(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTest1, self).setUp()
+    concurrency_model = TestModel1
+
+    def _get_target(self):
         self.TARGET = TestModel1(char_field="New", last_name="1")
 
     def test_force_update(self):
-        from django.db import connection, transaction, DatabaseError, IntegrityError
-
+        from django.db import DatabaseError
 
         t = self.TARGET.__class__()
         self.assertRaises(DatabaseError, t.save, force_update=True)
 
 
 class ConcurrencyTest2(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTest2, self).setUp()
+    concurrency_model = TestModel2
+
+    def _get_target(self):
         self.TARGET = TestModel2(char_field="New", last_name="1")
 
     def test_force_update(self):
-        from django.db import connection, transaction, DatabaseError, IntegrityError
-
+        from django.db import DatabaseError
 
         t = self.TARGET.__class__()
-        logger.debug("Object pk: %s version:%s", t.pk, t.version)
+        logger.debug("Object pk: %s version:%s", t.pk, t._get_test_revision_number())
         self.assertRaises(DatabaseError, t.save, force_update=True)
 
 
 class ConcurrencyTest3(ConcurrencyTest0):
+    concurrency_model = TestModel3
+
     def setUp(self):
         super(ConcurrencyTest3, self).setUp()
+
+    def _get_target(self):
         p, isnew = TestModel2.objects.get_or_create(char_field="New", last_name=str(time.time()))
         self.TARGET = TestModel3(char_field="New", last_name="1", fk=p)
 
     def test_force_update(self):
-        from django.db import connection, transaction, DatabaseError, IntegrityError
-
+        from django.db import DatabaseError
 
         t = self.TARGET.__class__()
         self.assertRaises(DatabaseError, t.save, force_update=True)
@@ -192,25 +235,26 @@ class ConcurrencyTest3(ConcurrencyTest0):
     def test_concurrency(self):
         a = self.TARGET
         self._check_save(a)
-        id = a.pk
+        a_pk = a.pk
 
-        a = self.TARGET.__class__.objects.get(pk=id)
-        v = a.version
-        logger.debug("reloaded...now version is %s " % a.version)
+        a = self.TARGET.__class__.objects.get(pk=a_pk)
+        v = a._get_test_revision_number()
+        logger.debug("reloaded...now version is %s " % v)
 
-        b = self.TARGET.__class__.objects.get(pk=id)
+        b = self.TARGET.__class__.objects.get(pk=a_pk)
         assert a.version == b.version, "got same row with different version"
         time.sleep(1)
         a.last_name = "pippo"
         self._check_save(a)
-        logger.debug("updated...now version is %s " % a.version)
-        assert a.version > v, "same or lower version after update (%s,%s)" % (a.version, v)
+        logger.debug("updated...now version is %s " % a._get_test_revision_number())
+        assert a.version > v, "same or lower version after update (%s,%s)" % (a._get_test_revision_number(), v)
         self.assertRaises(RecordModifiedError, b.save)
 
 
 class ConcurrencyTest4(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTest4, self).setUp()
+    concurrency_model = TestModel2
+
+    def _get_target(self):
         self.TARGET = TestModel2(char_field="New", last_name="1")
 
     def test_manager(self):
@@ -225,14 +269,10 @@ class ConcurrencyTest4(ConcurrencyTest0):
 
 
 class ConcurrencyTest0_Proxy(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTest0_Proxy, self).setUp()
+    def _get_target(self):
         self.TARGET = TestModel0_Proxy(char_field="New", last_name="1")
 
     def test_force_update(self):
-        from django.db import connection, transaction, DatabaseError, IntegrityError
-
-
         t = self.TARGET.__class__()
         t.save(force_update=True)
         t1 = TestModel0.objects.get(pk=t.pk)
@@ -240,24 +280,25 @@ class ConcurrencyTest0_Proxy(ConcurrencyTest0):
 
 
 class ConcurrencyTest2_Proxy(ConcurrencyTest2):
-    def setUp(self):
-        super(ConcurrencyTest2_Proxy, self).setUp()
+    def _get_target(self):
         self.TARGET = TestModel2(char_field="New", last_name="1")
 
 
 class ConcurrencyTest5(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTest5, self).setUp()
+    def _get_target(self):
         self.TARGET = TestAbstractModel0(char_field="New", last_name="1")
 
 
-class ConcurrencyTestModelUser(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTestModelUser, self).setUp()
-        self.TARGET = TestModelUser(username="ConcurrencyTestModelUser")
+class ConcurrencyTestTestCustomUser(ConcurrencyTest0):
+    concurrency_model = TestCustomUser
+    concurrency_kwargs = {'username': 'ConcurrencyTestTestCustomUser'}
+
+    def _get_target(self):
+        self.TARGET = TestCustomUser(username="ConcurrencyTestTestCustomUser")
 
     def _check_save(self, obj):
         obj.save()
+        assert obj.pk is not None # sanity check
         self.assertTrue(obj.version)
 
     def _get_form_data(self, **kwargs):
@@ -265,7 +306,25 @@ class ConcurrencyTestModelUser(ConcurrencyTest0):
             'last_login': datetime.datetime.today(),
             'date_joined': datetime.datetime.today(),
             'password': '123',
-            'username': 'ConcurrencyTestModelUser'}
+            'username': 'ConcurrencyTestTestCustomUser'}
+        data.update(**kwargs)
+        return data
+
+
+class ConcurrencyTestExistingModelUser(ConcurrencyTest0):
+    concurrency_model = User
+    concurrency_kwargs = {'username': 'test'}
+    apply_concurrency_check(User, 'version', IntegerVersionField)
+
+    def setUp(self):
+        super(ConcurrencyTestExistingModelUser, self).setUp()
+        self._unique_field_name = 'username'
+
+    def _get_target(self):
+        self.TARGET = User(username="aaa")
+
+    def _get_form_data(self, **kwargs):
+        data = {'username': 'aaaa', 'password': "123", 'last_login': '2000-01-01', 'date_joined': '2000-01-01'}
         data.update(**kwargs)
         return data
 
@@ -274,12 +333,13 @@ class ConcurrencyTestExistingModel(ConcurrencyTest0):
     """
 
     """
-    version = IntegerVersionField()
-    version.contribute_to_class(TestModelGroup, 'version')
+    apply_concurrency_check(TestModelGroup, 'version', IntegerVersionField)
 
     def setUp(self):
         super(ConcurrencyTestExistingModel, self).setUp()
         self._unique_field_name = 'name'
+
+    def _get_target(self):
         self.TARGET = TestModelGroup(name="aaa")
 
     def _get_form_data(self, **kwargs):
@@ -294,8 +354,7 @@ class ConcurrencyTestExistingModel(ConcurrencyTest0):
 
 
 class ConcurrencyTestModelWithCustomSave(ConcurrencyTest0):
-    def setUp(self):
-        super(ConcurrencyTestModelWithCustomSave, self).setUp()
+    def _get_target(self):
         self.TARGET = TestModelWithCustomSave(username="xxx", last_name="1")
 
     def _check_save(self, obj):
@@ -309,6 +368,8 @@ class ConcurrencyTestExistingModelWithCustomSave(ConcurrencyTest0):
     def setUp(self):
         super(ConcurrencyTestExistingModelWithCustomSave, self).setUp()
         self._unique_field_name = 'name'
+
+    def _get_target(self):
         self.TARGET = TestModelGroupWithCustomSave(name="aaa")
 
     def _check_save(self, obj):
@@ -328,7 +389,17 @@ class ConcurrencyTestExistingModelWithCustomSave(ConcurrencyTest0):
 
 
 class TestIssue3(ConcurrencyTest0):
-    def setUp(self):
-        super(TestIssue3, self).setUp()
+    def _get_target(self):
         self.TARGET = TestIssue3Model()
 
+
+class TestAbstractModelWithCustomSave(ConcurrencyTest0):
+    concurrency_model = ModelWithCustomSave
+
+    def _get_target(self):
+        self.TARGET = ModelWithCustomSave(username="New", last_name="1")
+
+    def _check_save(self, obj):
+        ret = obj.save()
+        self.assertSequenceEqual(ret, ('ModelWithCustomSave', 'AbstractModelWithCustomSave'))
+        self.assertTrue(obj.version)
