@@ -1,6 +1,11 @@
 from django.contrib.auth.models import User, Group
 from django.db import models
-from concurrency.fields import IntegerVersionField
+from django.db.models.base import ModelBase
+from concurrency.core import concurrency_check, _wrap_model_save
+from concurrency.fields import IntegerVersionField, AutoIncVersionField
+import logging
+
+logger = logging.getLogger('concurrency.test')
 
 
 class AbstractConcurrentModel(models.Model):
@@ -27,9 +32,64 @@ class ConcurrentModel(models.Model):
     class Meta:
         app_label = 'concurrency'
 
+    def __unicode__(self):
+        return "{0.__class__.__name__} #{0.pk}".format(self)
+
+
+class AutoIncConcurrentModel(models.Model):
+    version = AutoIncVersionField(db_column='cm_version_id')
+    username = models.CharField(max_length=30, blank=True, null=True)
+    last_name = models.CharField(max_length=30, blank=True, null=True)
+    char_field = models.CharField(max_length=30, blank=True, null=True)
+    date_field = models.DateField(blank=True, null=True)
+
+    class Meta:
+        app_label = 'concurrency'
+
+
+# class DateConcurrentModel(models.Model):
+#     version = DateTimeVersionField()
+#     username = models.CharField(max_length=30, blank=True, null=True)
+#     last_name = models.CharField(max_length=30, blank=True, null=True)
+#     char_field = models.CharField(max_length=30, blank=True, null=True)
+#     date_field = models.DateField(blank=True, null=True)
+#
+#     class Meta:
+#         app_label = 'concurrency'
+
+
+class MyMeta(ModelBase):
+    pass
+
+
+class AbstractModelWithCustomSave(models.Model):
+    __metaclass__ = MyMeta
+    version = IntegerVersionField(db_column='cm_version_id', manually=True)
+
+    class Meta:
+        abstract = True
+        app_label = 'concurrency'
+
+    def save(self, force_insert=False, force_update=False, using=None):
+        concurrency_check(self)
+        super(AbstractModelWithCustomSave, self).save(force_insert, force_update, using)
+        return 'AbstractModelWithCustomSave'
+
+
+class ModelWithCustomSave(AbstractModelWithCustomSave):
+    username = models.CharField(max_length=30, blank=True, null=True)
+    last_name = models.CharField(max_length=30, blank=True, null=True)
+
+    class Meta:
+        app_label = 'concurrency'
+
+    def save(self, force_insert=False, force_update=False, using=None):
+        ret = super(ModelWithCustomSave, self).save(force_insert, force_update, using)
+        return 'ModelWithCustomSave', ret
+
 
 class TestModel0(ConcurrentModel):
-    username = models.CharField(max_length=30, blank=True, null=True)
+    username = models.CharField(max_length=30, blank=True, null=True, unique=True)
     last_name = models.CharField(max_length=30, blank=True, null=True)
     char_field = models.CharField(max_length=30, blank=True, null=True)
     date_field = models.DateField(blank=True, null=True)
@@ -86,12 +146,15 @@ class TestModelWithCustomSave(ConcurrentModel):
         return 2222
 
 
-class TestModelUser(User):
+class TestCustomUser(User):
     version = IntegerVersionField(db_column='cm_version_id')
 
     class Meta:
         app_label = 'concurrency'
 
+    def __unicode__(self):
+        return "{0.__class__.__name__} #{0.pk}".format(self)
+
 
 class TestModelGroup(Group):
     #HACK: this field is here because all tests relies on that
@@ -105,7 +168,6 @@ class TestModelGroup(Group):
 
 
 class TestModelGroupWithCustomSave(TestModelGroup):
-
     class Meta:
         app_label = 'concurrency'
 
@@ -126,3 +188,9 @@ class TestIssue3Model(models.Model):
     class Meta:
         app_label = 'concurrency'
 
+
+# tox requires this.
+for model in [TestAbstractModel0, TestModel0, ModelWithCustomSave, TestIssue3Model, AutoIncConcurrentModel,
+              TestModelWithCustomSave]:
+    _wrap_model_save(model, True)
+
